unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Edit1: TEdit;
    btnTest: TButton;
    Label1: TLabel;
    btnEncode: TButton;
    btnDecode: TButton;
    ComboBox1: TComboBox;
    OpenDialog1: TOpenDialog;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label2: TLabel;
    Edit2: TEdit;
    procedure btnTestClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnEncodeClick(Sender: TObject);
    procedure btnDecodeClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  lastclock: int64=0;

implementation

uses SZCodeBaseX, Math;

{$R *.dfm}

function SZEncodeBase2(const S: string; MIMELines: integer=0): string;
{
 Example for binary Encoding
}
const
  Codes = '01';
  BITS = 1;
begin
  Result:=SZEncodeBaseXString(S, Codes, BITS, 0, MIMELines)
end;

function SZDecodeBase2(const S: string): string;
{
 Example for binary Decoding
}
const
  Codes = '01';
  BITS = 1;
begin
  Result:=SZDecodeBaseXString(S, Codes, BITS)
end;

procedure TForm1.btnTestClick(Sender: TObject);
var
  s,e,d: string;

  procedure PrintNote(const s, e, d: string);
  begin
    with Memo1.Lines do
    begin
      Add(s);
      Add('Encode: ' + e);
      Add('Decode: ' + d);
      Add('');
    end
  end;

begin
  Memo1.Clear;

  s:=Edit1.Text;

  with Memo1.Lines do
  begin
    Add('Text for testing: ' + s);
    Add('');

    e:=SZEncodeBase16(s); d:=SZDecodeBase16(e);
    PrintNote('Base16',e,d);

    e:=SZEncodeBase32(s); d:=SZDecodeBase32(e);
    PrintNote('Base32 - RFC 3548 incompatibility - no padding keys',e,d);

    e:=SZFullEncodeBase32(s); d:=SZDecodeBase32(e);
    PrintNote('Base32 - RFC 3548 full compatibility',e,d);

    e:=SZEncodeBase64(s); d:=SZDecodeBase64(e);
    PrintNote('Base64 - RFC 3548 incompatibility - no padding keys',e,d);

    e:=SZFullEncodeBase64(s); d:=SZDecodeBase64(e);
    PrintNote('Base64 - RFC 3548 full compatibility',e,d);

    e:=SZEncodeBase64URL(s); d:=SZDecodeBase64URL(e);
    PrintNote('Base64URL - RFC 3548 incompatibility - no padding keys',e,d);

    e:=SZFullEncodeBase64URL(s); d:=SZDecodeBase64URL(e);
    PrintNote('Base64URL - RFC 3548 full compatibility',e,d);

    e:=SZEncodeBase2(s); d:=SZDecodeBase2(e);
    PrintNote('Base2 - Example how to create your own base Encode/Decode functions',e,d);

   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  OpenDialog1.FileName:='.\';
end;

procedure TForm1.btnEncodeClick(Sender: TObject);
var
  e,d: string;
  Mime,i: integer;
begin

{
  d:='';
  for i:=0 to memo1.Lines.count-1 do
    d:=d+memo1.Lines[i]+#13#10;
}
d:=Memo1.Text;

  Mime:= StrToIntDef(Edit2.Text,0);
  if Mime<0 then
     Mime:=0;
  
  case ComboBox1.ItemIndex of
    0: e:=SZEncodeBase16(d,Mime);
    1: e:=SZEncodeBase32(d,Mime);
    2: e:=SZEncodeBase64(d,Mime);
    3: e:=SZEncodeBase64URL(d,Mime);
    4: e:=SZEncodeBase2(d,Mime);
    5: e:=SZFullEncodeBase32(d,Mime);
    6: e:=SZFullEncodeBase64(d,Mime);
    7: e:=SZFullEncodeBase64URL(d,Mime);
  end;
  //Edit2.Text:=e;
  memo1.text:=e
end;

procedure TForm1.btnDecodeClick(Sender: TObject);
var
  e,d: string;
  i: integer;
begin
  e:='';
  for i:=0 to memo1.Lines.count-1 do
    e:=e+memo1.Lines[i]+#13#10;


  case ComboBox1.ItemIndex of
    0: d:=SZDecodeBase16(e);
    1: d:=SZDecodeBase32(e);
    2: d:=SZDecodeBase64(e);
    3: d:=SZDecodeBase64URL(e);
    4: d:=SZDecodeBase2(e);
    5: d:=SZDecodeBase32(e);
    6: d:=SZDecodeBase64(e);
    7: d:=SZDecodeBase64URL(e);
  end;
//  Edit1.Text:=d;
  memo1.text:=d
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  OpenDialog1.FileName:=Edit1.Text;
  if OpenDialog1.Execute then
  begin
    Edit1.Text:=OpenDialog1.FileName;
  end
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  sIn,sOut: TFileStream;
  fIn,fOut: String;
  size: integer;
begin
  fIn := edit1.Text;
  fOut:='Encode.tmp';

{
  // Encoding directly by given filename
  sOut:=TFileStream.Create(fOut,fmCreate);
  size:=SZFullEncodeBase64(fIn, sOut);
  sOut.Free;
}

  // Stream to stream
  sIn :=TFileStream.Create(fIn, fmOpenRead or fmShareDenyNone);
  sOut:=TFileStream.Create(fOut, fmCreate);

  // Full Base64 encoding - with padding keys
  size:=SZFullEncodeBase64(sIn, sOut, -1, StrToIntDef(Edit2.Text,0));

  memo1.Lines.Add(
    format('Encoded "%s" (%d bytes) to "%s" (%d bytes) ',
      [fIn,sIn.Size,fOut,sOut.Size]));

  sIn.Free;
  sOut.Free;    
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  sIn,sOut: TFileStream;
  fIn,fOut: String;
  size: integer;
begin

  fIn :='Encode.tmp';
  fOut:='Decode.tmp';

// Stream to stream
  sIn :=TFileStream.Create(fIn, fmOpenRead or fmShareDenyNone);
  sOut:=TFileStream.Create(fOut, fmCreate);

  // Decode Base64
  size:=SZDecodeBase64(sIn, sOut);

  memo1.Lines.Add(
    format('Decoded "%s" (%d bytes) to "%s" (%d bytes) ',
      [fIn,sIn.Size,fOut,sOut.Size]));

  sIn.Free;
  sOut.Free;

end;

end.
